@@ -1,7 +1,9 @@
 #!/usr/pkg/bin/python3.11
 # Impacket - Collection of Python classes for working with network protocols.
 #
-# Copyright (C) 2023 Fortra. All rights reserved.
+# Copyright Fortra, LLC and its affiliated companies 
+#
+# All rights reserved.
 #
 # This software is provided under a slightly modified version
 # of the Apache Software License. See the accompanying LICENSE file
@@ -34,39 +36,13 @@ import struct
 import argparse
 import cmd
 import ntpath
-# If you wanna have readline like functionality in Windows, install pyreadline
-try:
-  import pyreadline as readline
-except ImportError:
-  import readline
 from six import PY2, text_type
 from datetime import datetime
 from impacket.examples import logger
 from impacket import version
-from impacket.structure import Structure
+from impacket.structure import Structure, hexdump
 
 
-def pretty_print(x):
-    visible = '0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ!"#$%&\'()*+,-./:;<=>?@[\\]^_`{|}~ '
-    return x if x in visible else '.'
-
-def hexdump(data):
-    x = str(data)
-    strLen = len(x)
-    i = 0
-    while i < strLen:
-        print("%04x  " % i, end=' ')
-        for j in range(16):
-            if i+j < strLen:
-                print("%02X" % ord(x[i+j]), end=' ')
-            else:
-                print("  ", end=' ')
-            if j%16 == 7:
-                print("", end=' ')
-        print(" ", end=' ')
-        print(''.join(pretty_print(x) for x in x[i:i+16] ))
-        i += 16
-
 # Reserved/fixed MFTs
 FIXED_MFTS = 16
 
@@ -1020,7 +996,7 @@ class MiniShell(cmd.Cmd):
         cmd.Cmd.__init__(self)
         self.volumePath = volume
         self.volume = NTFS(volume)
-        self.rootINode = self.volume.getINode(5)
+        self.rootINode = self.volume.getINode(FILE_Root)
         self.prompt = '\\>'
         self.intro = 'Type help for list of commands'
         self.currentINode = self.rootINode
@@ -1198,24 +1174,18 @@ class MiniShell(cmd.Cmd):
 
 def main():
     print(version.BANNER)
-    # Init the example's logger theme
-    logger.init()
     parser = argparse.ArgumentParser(add_help = True, description = "NTFS explorer (read-only)")
     parser.add_argument('volume', action='store', help='NTFS volume to open (e.g. \\\\.\\C: or /dev/disk1s1)')
     parser.add_argument('-extract', action='store', help='extracts pathname (e.g. \\windows\\system32\\config\\sam)')
     parser.add_argument('-debug', action='store_true', help='Turn DEBUG output ON')
+    parser.add_argument('-ts', action='store_true', help='Adds timestamp to every logging output')
 
     if len(sys.argv)==1:
         parser.print_help()
         sys.exit(1)
     options = parser.parse_args()
-
-    if options.debug is True:
-        logging.getLogger().setLevel(logging.DEBUG)
-        # Print the Library's installation path
-        logging.debug(version.getInstallationPath())
-    else:
-        logging.getLogger().setLevel(logging.INFO)
+    # Init the example's logger theme
+    logger.init(options.ts, options.debug)
 
     shell = MiniShell(options.volume)
     if options.extract is not None:
